<?php

namespace Config;

// Create a new instance of our RouteCollection class.
$routes = Services::routes();

// Load the system's routing file first, so that the app and ENVIRONMENT
// can override as needed.
if (file_exists(SYSTEMPATH . 'Config/Routes.php')) {
  require SYSTEMPATH . 'Config/Routes.php';
}

/**
 * --------------------------------------------------------------------
 * Router Setup
 * --------------------------------------------------------------------
 */
$routes->setDefaultNamespace('App\Controllers');
$routes->setDefaultController('Dashboard');
$routes->setDefaultMethod('index');
$routes->setTranslateURIDashes(false);
$routes->set404Override();
$routes->setAutoRoute(true);

/*
 * --------------------------------------------------------------------
 * Route Definitions
 * --------------------------------------------------------------------
 */

// We get a performance increase by specifying the default
// route since we don't have to scan directories.
$routes->get('/', 'Dashboard::index');

// dashboard routes
$routes->post('/submit-capacity', 'Dashboard::myCapacity');
$routes->post('/submit-next-week-capacity', 'Dashboard::nextWeekCapacity');
$routes->post('/set-status', 'Dashboard::setStatus');

// user authentication routes
$routes->get('/login', 'Auth::login');
$routes->get('/logout', 'Auth::logout');
$routes->get('/not-authorized', 'Auth::notAuthorized');
$routes->get('/invalid-state', 'Auth::invalidState');
$routes->get('/refresh-session', 'Auth::refreshSession');

// user profile routes
$routes->get('/profile', 'User::showProfile');
$routes->post('/submit-interests/(:any)', 'User::myInterests/$1');

// user capacity routes
$routes->get('/capacity/history', 'User::viewCapacityHistory');
$routes->get('/capacity/report', 'User::viewCapacityReport');

// user asignment routes
$routes->get('/assignments/new', 'User::viewNewAssignments');
$routes->get('/assignments/current', 'User::viewCurrentAssignments');
$routes->get('/assignments/report', 'User::viewAssignmentReport');

// admin routes
$routes->get('/admin', 'Admin::index');
$routes->get('/admin/show-capacity/(:any)', 'Admin::showCapacity/$1');
$routes->get('/admin/list-associates', 'Admin::listAssociates');
$routes->get('/admin/view-profile/(:any)', 'Admin::viewProfile/$1');

// parter routes
$routes->get('/partner', 'Partner::index'); // partner dashboard
$routes->get('/partner/tasks/(:any)/(:num)', 'Partner::tasks/$1/$2'); // new, assigned, reports
$routes->get('/partner/matters/(:any)/(:num)', 'Partner::matters/$1/$2'); //
$routes->get('/partner/associates/(:any)/(:num)', 'Partner::associates/$1/$2'); //
$routes->post('/partner/reports', 'Partner::reports'); //

// API routes
$routes->resource('capacity');

/*
 * --------------------------------------------------------------------
 * Additional Routing
 * --------------------------------------------------------------------
 *
 * There will often be times that you need additional routing and you
 * need it to be able to override any defaults in this file. Environment
 * based routes is one such time. require() additional route files here
 * to make that happen.
 *
 * You will have access to the $routes object within that file without
 * needing to reload it.
 */
if (file_exists(APPPATH . 'Config/' . ENVIRONMENT . '/Routes.php')) {
  require APPPATH . 'Config/' . ENVIRONMENT . '/Routes.php';
}
