<?php

namespace App\Controllers;

use App\Models\CapacityModel;
use App\Models\PartnersOfInterestModel;
use App\Models\MyPracticeAreasModel;
use App\Models\MyExperiencesModel;
use App\Models\UsersModel;

use CodeIgniter\Controller;

$session = session();

class Admin extends Controller
{
  public function index()
  {
    
    if(!isset($_SESSION['oauth2state'])) {
      if(!isset($_GET['cipher'])){
        return redirect()->to('https://toolkit.africalegalnetwork.com');
      }else{
        $_SESSION['user_email'] = $_GET['email'];
        return redirect()->to('/login?cipher=' . $_GET['cipher'] . '&email=' . $_GET['email']);
      }
    }
    
    // generate app tokens
    // 1. FAQs
    $faqsToken = bin2hex(random_bytes(16));
    
    //encrypt token
    $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
    $iv = openssl_random_pseudo_bytes($ivlen);
    $ciphertext_raw = openssl_encrypt($faqsToken, $cipher, $_ENV['FAQS_RELAY_KEY'], $options=OPENSSL_RAW_DATA, $iv);
    $hmac = hash_hmac('sha256', $ciphertext_raw, $_ENV['FAQS_RELAY_KEY'], $as_binary=true);
    $faqsCipher = base64_encode( $iv.$hmac.$ciphertext_raw );
    
    // 2. Deal Room
    $dealRoomToken = bin2hex(random_bytes(16));
    
    //encrypt token
    $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
    $iv = openssl_random_pseudo_bytes($ivlen);
    $ciphertext_raw = openssl_encrypt($dealRoomToken, $cipher, $_ENV['DEAL_ROOM_RELAY_KEY'], $options=OPENSSL_RAW_DATA, $iv);
    $hmac = hash_hmac('sha256', $ciphertext_raw, $_ENV['DEAL_ROOM_RELAY_KEY'], $as_binary=true);
    $dealRoomCipher = base64_encode( $iv.$hmac.$ciphertext_raw );
    
    $_SESSION['faqsCipher'] = urlencode($faqsCipher);
    $_SESSION['dealRoomCipher'] = urlencode($dealRoomCipher);
    // end token generation

    $capacity_model = new CapacityModel();
    $weeks_capacity = $capacity_model->getWeeksCapacity(null, null);

    $data = ['weeks_capacity' => $weeks_capacity];

    echo view('user/admin/header', $_SESSION);
    echo view('user/admin/dashboard', $data);
    echo view('user/admin/footer');
  }

  // list associates
  public function listAssociates()
  {
    $users_model = new UsersModel();
    $associates = $users_model->listAssociates(null, null);

    $data = ['associates' => $associates];

    echo view('user/admin/header');
    echo view('user/admin/list-associates', $data);
    echo view('user/admin/footer');
  }

  public function showCapacity($day)
  {
    $capacity_model = new CapacityModel();
    $capacity = $capacity_model->getDaysCapacity($day);

    $date = date_create(date('Y-m-d'));
    $today = strtolower(date_format($date, 'l'));

    $data = [
      'day' => $day,
      'capacity' => $capacity,
      'today' => $today,
    ];

    echo view('user/admin/header', $data);
    echo view('user/admin/capacity', $data);
    echo view('user/admin/footer');
  }

  public function viewProfile($employeeNumber)
  {
    $users_model = new UsersModel();

    $user = $users_model->getProfileDetails($employeeNumber);
//     $interests = $users_model->getInterests($employeeNumber);
//     $experience = $users_model->getExperience($employeeNumber);
// 
//     $areas = json_decode($interests['areas_of_interest']);
//     $partners = json_decode($interests['partners']);

//     if (!$areas) {
//       $areas = [];
//     }
// 
//     if (!$partners) {
//       $partners = [];
//     }
    
    $data = [
      'name' => $user['name'],
      'email' => $user['email'],
      'status' => $user['status'],
      'level' => $user['level'],
      'department' => $user['department'],
      // 'areas' => $areas,
      // 'partners' => $partners,
      // 'experiences' => $experience,
    ];

    echo view('user/admin/header');
    echo view('user/admin/user-profile', $data);
    echo view('user/admin/footer');
  }
}
