<?php

namespace App\Controllers;

use App\Models\UsersModel;

use CodeIgniter\Controller;

$session = session();

class Auth extends BaseController
{
  public function login()
  {
    $users_model = new UsersModel();

    $cipherText = str_replace(' ', '+', urldecode($_GET['cipher']));
    $email = urldecode($_GET['email']);
    
    // decrypt the token
    $c = base64_decode($cipherText);
    $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
    $iv = substr($c, 0, $ivlen);
    $hmac = substr($c, $ivlen, $sha2len=32);
    $ciphertext_raw = substr($c, $ivlen+$sha2len);
    $token = openssl_decrypt($ciphertext_raw, $cipher, $_ENV['RELAY_KEY'], $options=OPENSSL_RAW_DATA, $iv);
    $calcmac = hash_hmac('sha256', $ciphertext_raw, $_ENV['RELAY_KEY'], $as_binary=true);
    
    if($_ENV['CI_ENVIRONMENT'] != 'development'){
      if (!hash_equals($hmac, $calcmac)) return redirect()->to('/invalid-state');
    }
    
    $user = $users_model->getByEmail($email);
    
    // if user doesn't exist, kick them out
    if (!$user['employeeNumber']) return redirect()->to('/not-authorized');
    
    // 4. add user details to the session
    $_SESSION['user_employeeNumber'] = $user['employeeNumber'];
    $_SESSION['user_name'] = $user['name'];
    $_SESSION['user_email'] = $email;
    $_SESSION['user_id_org_level'] = $user['id_org_level'];
    $_SESSION['user_id_role'] = $user['id_role'];
    $_SESSION['user_id_department'] = $user['id_department'];
    $_SESSION['user_status'] = $user['status'];
    $_SESSION['oauth2state'] = $token;

    // set cookie
    setcookie('userData', json_encode($_SESSION));

    // if user id is not set, have the user log in
    if(!$user) return redirect()->to('/login');

    // 5. if user is admin
    if ($user['id_role'] == 1) return redirect()->to('/admin');

    // 6. if user is partner
    if ($user['id_role'] == 2) return redirect()->to('/partner');
    
    // 7. if user is associate
    if ($user['id_role'] == 3) return redirect()->to('/');

    // user data is blank
    session_destroy();
    return redirect()->to('https://toolkit.africalegalnetwork.com');
  }

  // invalid state
  public function invalidState()
  {
    //unset session
    session_destroy();

    // clear cookies
    setcookie('userData', '', 1, '/'); // empty value and old timestamp
    setcookie('myPartners', '', 1, '/');
    setcookie('capacityToday', '', 1, '/');
    setcookie('partners', '', 1, '/');
    setcookie('practiceAreas', '', 1, '/');
    setcookie('myPracticeAreas', '', 1, '/');
    setcookie('myExperience', '', 1, '/');
    setcookie('areasOfInterest', '', 1, '/');
    setcookie('myAreasOfInterest', '', 1, '/');

    echo view('user/invalid-state');
  }

  // unauthorised access
  public function notAuthorized()
  {
    echo view('user/not-authorized');
  }

  // log out
  public function logout()
  {
    session_destroy();

    // clear cookies
    setcookie('userData', '', 1, '/'); // empty value and old timestamp
    setcookie('myPartners', '', 1, '/');
    setcookie('capacityToday', '', 1, '/');
    setcookie('partners', '', 1, '/');
    setcookie('practiceAreas', '', 1, '/');
    setcookie('myPracticeAreas', '', 1, '/');
    setcookie('myExperience', '', 1, '/');
    setcookie('areasOfInterest', '', 1, '/');
    setcookie('myAreasOfInterest', '', 1, '/');

    // clear localStorage and local cookies
    echo view('clear-data');
    
    if($_ENV['CI_ENVIRONMENT'] != 'development') return redirect()->to('https://toolkit.africalegalnetwork.com');

    echo view('user/logged-out');
  }

  // refresh session
  public function refreshSession()
  {
    echo 'Session refreshed';
  }
}
