<?php

namespace App\Controllers;

use App\Models\CapacityModel;
use App\Models\NextWeekCapacityModel;
use App\Models\AssignmentsModel;
use App\Models\UsersModel;


$session = session();

class Dashboard extends BaseController
{
  public function index()
  {
    $capacity_model = new CapacityModel();
    $nextWeekCapacityModel = new NextWeekCapacityModel();
    $assignments_model = new AssignmentsModel();
    
    if($_ENV['CI_ENVIRONMENT'] != 'development'){
      if(!isset($_SESSION['oauth2state'])) {
        if(!isset($_GET['cipher'])){
          return redirect()->to('https://toolkit.africalegalnetwork.com');
        }else{
          $_SESSION['user_email'] = $_GET['email'];
          return redirect()->to('/login?cipher=' . $_GET['cipher'] . '&email=' . $_GET['email']);
        }
      }
    }else{ // this block is used to test the app locally
      if(!isset($_SESSION['oauth2state'])) {
        return redirect()->to('/login?cipher=qlIKRnKw667EzecTxXtKoexGQaMN6cRT&email=Nahabi.Wandera@aln.africa');
      }
    }
    
    // generate app tokens
    // 1. FAQs
    $faqsToken = bin2hex(random_bytes(16));
    
    //encrypt token
    $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
    $iv = openssl_random_pseudo_bytes($ivlen);
    $ciphertext_raw = openssl_encrypt($faqsToken, $cipher, $_ENV['FAQS_RELAY_KEY'], $options=OPENSSL_RAW_DATA, $iv);
    $hmac = hash_hmac('sha256', $ciphertext_raw, $_ENV['FAQS_RELAY_KEY'], $as_binary=true);
    $faqsCipher = base64_encode( $iv.$hmac.$ciphertext_raw );
    
    // 2. Deal Room
    $dealRoomToken = bin2hex(random_bytes(16));
    
    //encrypt token
    $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
    $iv = openssl_random_pseudo_bytes($ivlen);
    $ciphertext_raw = openssl_encrypt($dealRoomToken, $cipher, $_ENV['DEAL_ROOM_RELAY_KEY'], $options=OPENSSL_RAW_DATA, $iv);
    $hmac = hash_hmac('sha256', $ciphertext_raw, $_ENV['DEAL_ROOM_RELAY_KEY'], $as_binary=true);
    $dealRoomCipher = base64_encode( $iv.$hmac.$ciphertext_raw );
    
    $_SESSION['faqsCipher'] = urlencode($faqsCipher);
    $_SESSION['dealRoomCipher'] = urlencode($dealRoomCipher);
    // end token generation
    
    // get today's capacity
    $capacity_today = $capacity_model->getTodaysCapacity($_SESSION['user_employeeNumber']);
    
    $data = [
      'capacity' => $capacity_model->getMyCapacity($_SESSION['user_employeeNumber']),
      'nextWeekCapacity' => $nextWeekCapacityModel->getMyCapacity($_SESSION['user_employeeNumber']),
      'capacity_today' => $capacity_today['capacity'],
      'assignments' => $assignments_model->getAssociateAssignments($_SESSION['user_employeeNumber'])
    ];

    echo view('templates/header', $_SESSION);
    echo view('dashboard/dashboard', $data);
    echo view('templates/footer');
  }

  public function myCapacity()
  {
    $capacity_model = new CapacityModel();

    $capacity = json_decode($this->request->getPost('capacity'));

    foreach ($capacity as $my_capacity) {
      $capacity_value = $my_capacity->value;
      $capacity_id = $my_capacity->id;

      $capacity_model->save([
        'id' => $capacity_id,
        'capacity' => $capacity_value,
      ]);
    }

    echo '<p>Capacity updated</p>';
  }
  
  public function nextWeekCapacity()
  {
    $nextWeekCapacityModel = new NextWeekCapacityModel();

    $capacity = $this->request->getPost('capacity');
    $id = $this->request->getPost('id');

    $nextWeekCapacityModel->save([
      'id' => $id,
      'capacity' => $capacity
    ]);

    echo '<p>Capacity updated</p>';
  }
  
  public function setStatus(){
    $usersModel = new UsersModel();
    
    $status = $this->request->getPost('status');
    $employeeNumber = $this->request->getPost('employeeNumber');
    
    $usersModel->set('status', $status);
    $usersModel->where('employeeNumber', $employeeNumber);
    $usersModel->update();
    
    echo '<p>Status updated</p>';
  }
}
