<?php

namespace App\Controllers;

use App\Models\MattersModel;
use App\Models\MatterPartnersModel;
use App\Models\AssignmentsModel;
use App\Models\UsersModel;
use App\Models\CapacityModel;
use App\Models\AssociateInterestsModel;
use App\Models\DepartmentsModel;

use CodeIgniter\Controller;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\Exception;

require ROOTPATH . 'vendor/autoload.php';

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;

$session = session();

class Partner extends Controller
{
  public function index()
  {
    
    if($_ENV['CI_ENVIRONMENT'] != 'development'){
      if(!isset($_SESSION['oauth2state'])) {
        return redirect()->to('https://toolkit.africalegalnetwork.com');
      }
    }else{ // this block is used to test the app locally
      if(!isset($_SESSION['oauth2state'])) {
        return redirect()->to('/login?cipher=qlIKRnKw667EzecTxXtKoexGQaMN6cRT&email=nahabi@africalegalnetwork.com');
      }
    }
    
    // generate app tokens
    // 1. FAQs
    $faqsToken = bin2hex(random_bytes(16));
    
    //encrypt token
    $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
    $iv = openssl_random_pseudo_bytes($ivlen);
    $ciphertext_raw = openssl_encrypt($faqsToken, $cipher, $_ENV['FAQS_RELAY_KEY'], $options=OPENSSL_RAW_DATA, $iv);
    $hmac = hash_hmac('sha256', $ciphertext_raw, $_ENV['FAQS_RELAY_KEY'], $as_binary=true);
    $faqsCipher = base64_encode( $iv.$hmac.$ciphertext_raw );
    
    // 2. Deal Room
    $dealRoomToken = bin2hex(random_bytes(16));
    
    //encrypt token
    $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
    $iv = openssl_random_pseudo_bytes($ivlen);
    $ciphertext_raw = openssl_encrypt($dealRoomToken, $cipher, $_ENV['DEAL_ROOM_RELAY_KEY'], $options=OPENSSL_RAW_DATA, $iv);
    $hmac = hash_hmac('sha256', $ciphertext_raw, $_ENV['DEAL_ROOM_RELAY_KEY'], $as_binary=true);
    $dealRoomCipher = base64_encode( $iv.$hmac.$ciphertext_raw );
    
    $_SESSION['faqsCipher'] = urlencode($faqsCipher);
    $_SESSION['dealRoomCipher'] = urlencode($dealRoomCipher);
    // end token generation
    
    $capacity_model = new CapacityModel();
    $users_model = new UsersModel();
    $matters_model = new MattersModel();
    $departments_model = new DepartmentsModel();

    $averages = $capacity_model->getDeptWeeklyAverages($_SESSION['user_id_department']);
    $no_of_associates = count($capacity_model->getDashboardData($_SESSION['user_id_department']));
    // $assignments = $matters_model->getAssignments($_SESSION['user_id_department'], $_SESSION['user_employeeNumber']); // use this when there's more than one assigning partner
    $assignments = $matters_model->getAssignments($_SESSION['user_id_department']);
    $department = $departments_model->getOne($_SESSION['user_id_department']);

    $data = [
      'averages' => json_encode($averages),
      'associates' => $no_of_associates,
      'assignments' => $assignments,
      'department' => $department['department'],
      'partners' => [],
      'matters' => [],
    ];
    
    $headerData = $_SESSION;
    
    $headerData['department'] = $department['department'];
    $headerData['averages'] = $capacity_model->getHeaderAverages($_SESSION['user_id_department'])[0];
    $headerData['associates'] = $no_of_associates;

    echo view('user/partner/header', $headerData);
    echo view('user/partner/dashboard', $data);
    echo view('user/partner/footer');
  }

  // matters
  public function matters($fn, $id = null)
  {
    $matters_model = new MattersModel();
    $assignments_model = new AssignmentsModel();
    $users_model = new UsersModel();

    helper('form');

    switch ($fn) {
      // submit new matter
      case 'submit':
        echo '<p>Submit matter</p>';
        break;

      // edit matter
      case 'edit':
        echo '<p>Edit matter</p>';
        break;

      // delete matter
      case 'delete':
        echo '<p>Matter deleted</p>';
        break;

      // show matters list
      default:
        $data = [
          'matters' => $matters_model->getMyMatters($_SESSION['user_employeeNumber']),
          'assignments' => $assignments_model->getPartnerAssignments($_SESSION['user_employeeNumber']),
          'associates' => $users_model->listAssociates(null, null),
          'partners' => [],
        ];

        echo view('user/partner/header');
        echo view('user/partner/matters/' . $fn, $data);
        echo view('user/partner/footer');

        break;
    }
  }

  // associates
  public function associates($fn, $id = null)
  {
    $users_model = new UsersModel();
    $capacity_model = new CapacityModel();
    $interests_model = new AssociateInterestsModel();
    $matters_model = new MattersModel();
    $matter_partners_model = new MatterPartnersModel();
    $assignments_model = new AssignmentsModel();
    $departments_model = new DepartmentsModel();

    $associates = $users_model->listAssociates($_SESSION['user_id_department'], null);

    //Instantiation and passing `true` enables exceptions
    $mail = new PHPMailer(true);

    //Server settings
    // $mail->SMTPDebug = SMTP::DEBUG_SERVER; //Enable verbose debug output
    $mail->CharSet = 'UTF-8'; // character encoding
    $mail->isSMTP(); //Send using SMTP
    $mail->Host = 'smtp.sendgrid.net'; //Set the SMTP server to send through
    $mail->SMTPAuth = true; //Enable SMTP authentication
    $mail->Username = 'apikey'; //SMTP username
    $mail->Password = $_ENV['SG_API_KEY']; //SMTP password
    $mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS; //Enable TLS encryption; `PHPMailer::ENCRYPTION_SMTPS` encouraged
    $mail->Port = 465; //TCP port to connect to, use 465 for `PHPMailer::ENCRYPTION_SMTPS` above

    //Content
    $mail->isHTML(true); //Set email format to HTML
    $mail->setFrom('capacity@africalegalnetwork.com', 'Capacity Log');
    $mail->addReplyTo('capacity@africalegalnetwork.com', 'Capacity Log');

    switch ($fn) {
      case 'submit-assignment':
        $assignment = json_decode($_POST['assignment']);
        
        // define empty partner emails array
        $partners_to_email = [];

        $email_data = [
          'message' => $assignment->message,
          'partners' => [],
          'associates' => [],
        ];

        // if this is a new matter, add it to the db and get its id
        if ($assignment->newMatter->requirement) {
          $new_matter = [
            'requirement' => $assignment->newMatter->requirement,
            'details' => $assignment->newMatter->details,
            'added_by' => $_SESSION['user_employeeNumber'],
          ];

          $matters_model->transStart();
          $id_matter = $matters_model->insert($new_matter);
          $matters_model->transComplete();

          // populate email data for partner
          $email_data['details'] = $assignment->newMatter->details;
          $email_data['requirement'] = $assignment->newMatter->requirement;
          $email_data['assignmentMessage'] = $assignment->message;

          // save matter partners
          $partners = $assignment->newMatter->partners;

          foreach ($partners as $partner_id) {
            $matter_data = [
              'id_matter' => $id_matter,
              'id_partner' => $partner_id,
            ];

            $matter_partners_model->insert($matter_data);

            // get partner details for inclusion in email
            $partner = $users_model->getByEmployeeNumber($partner_id);
            array_push($email_data['partners'], $partner['name']);

            // push partner emails to partners to email array
            array_push($partners_to_email, $partner['email']);
          }
        } else {
          $id_matter = $assignment->existingMatter;

          // get matter details to include in email
          $matter = $matters_model->getOne($id_matter);

          $email_data['details'] = $matter['details'];

          // get matter partners
          $matter_partner_ids = $matter_partners_model->getPartnersByMatterId(
            $id_matter
          );

          foreach ($matter_partner_ids as $partner_id) {
            $partner = $users_model->getOne($partner_id['id_partner']);
            array_push($email_data['partners'], $partner['name']);

            // push partner emails to partners to email array
            array_push($partners_to_email, $partner['email']);
          }
        }

        // create associate email message
        $email_message = view('emails/to-associate', $email_data);

        // make the assignment
        foreach ($assignment->assignees as $assignee) {
          $assignment_data = [
            'id_matter' => $id_matter,
            'assignment_message' => $assignment->message,
            'assigned_to' => $assignee->id,
            'assigned_by' => $_SESSION['user_employeeNumber'],
          ];

          $assignments_model->save($assignment_data);

          // add associate details to email message
          array_push($email_data['associates'], $assignee->name);

          // send email to associate
          // get associate's email
          $associate = $users_model->getByEmployeeNumber($assignee->id);

          try {
            $mail->Subject = 'New Assignment';
            //Recipients
            // $mail->addAddress('Nahabi.Wandera@aln.africa'); //Name is optional
            $mail->addAddress($associate['email'], $associate['name']); //Name is optional

            $mail->Body = $email_message;
            // $mail->AltBody = 'This is the body in plain text for non-HTML mail clients';

            // if($ENV['CI_ENVIRONMENT'] != 'development'){
              $mail->send();
            // }
          } catch (Exception $e) {
            echo 'Assignment failed';
            echo $mail->ErrorInfo;
          }

          // send email to partners
        }

        // create partner email message
        $partner_email_message = view('emails/to-partner', $email_data);

        // send email to partners
        // get partner details from db
        foreach ($partners_to_email as $partner_email) {
          $mail->Subject = 'New assignment made';
          //Recipients
          $mail->addAddress($partner_email); //Name is optional

          $mail->Body = $partner_email_message;
          
          // if($ENV['CI_ENVIRONMENT'] != 'development'){
            $mail->send();
          // }
        }

        // send message to person making the assignment
        $mail->addAddress($_SESSION['user_email'], $_SESSION['user_name']); //Name is optional
        // $mail->addAddress('nahabi@africalegalnetwork.com');
        $mail->addBCC('Nahabi.Wandera@aln.africa', 'Nahabi Wandera');
        $mail->Subject = 'New assignment';
        $mail->Body = $partner_email_message;
        
        // if($ENV['CI_ENVIRONMENT'] != 'development'){
          $mail->send();
        // }

        // confirm assignment to user
        echo '<p>Assignment made</p>';

        break;

      case 'getsingle':
        $associate = $users_model->listAssociates(null, $id);
        $capacity = $capacity_model->getMycapacity($id);
        $interests = $interests_model->getMyInterests($id);
        $assignments = $assignments_model->getAssociateAssignments($id);
        $totals = $capacity_model->getCapacityBillables($id);
        
        $data = [
          'associate' => $associate,
          'capacity' => $capacity,
          'interests' => $interests,
          'assignments' => $assignments,
          'totals' => json_encode($totals),
        ];

        echo view('user/partner/associates/single', $data);
        break;
        
      case 'delete-assignment':
        // reassign matter
        $assignments_model->set('reassigned', 'Y');
        $assignments_model->set('date_reassigned', date('Y-m-d H:i:s'));
        $assignments_model->where('id', $id);
        $assignments_model->update();
        
        // get matter id
        $assignment = $assignments_model->getOne($id);
        $matterId = $assignment['id_matter'];
        
        // get matter
        $matter = $matters_model->getOne($matterId);
        
        // get associate details
        $associate = $users_model->getOne($assignment['assigned_to']);
        
        // get partners
        $partners = $matters_model->getMatterPartners($id);
        
        $email_data = ['details' => $matter['details'], 'associate' => $associate['name']];
        
        
        // create email messages
        $associate_email_message = view('emails/reassign-associate', $email_data);
        $partner_email_message = view('emails/reassign-partner', $email_data);
        
        
        // send email to associate
        try {
          $mail->Subject = 'Matter reassigned';
        
          //Recipients
          // $mail->addAddress('nahabi@africalegalnetwork.com'); //Name is optional
          $mail->addAddress($associate['email'], $associate['name']); //Name is optional
          $mail->Body = $associate_email_message;
          $mail->send();
        
        } catch (Exception $e) {
          echo 'Reassignment email not sent';
          echo $mail->ErrorInfo;
        }
        
        // send email to partners
        foreach ($partners as $partner) {
          $mail->Subject = 'Matter reassignment';
          //Recipients
          $mail->addAddress($partner['email']); //Name is optional
          $mail->Body = $partner_email_message;
          
          $mail->send();
        }
        
        // send email to assignor
         $mail->Subject = 'Matter reassignment';
         //Recipients
         // $mail->addAddress('Nahabi.Wandera@aln.africa');
         $mail->addAddress($_SESSION['user_email'], $_SESSION['user_name']);
         $mail->Body = $partner_email_message;
         
         $mail->send();
        
        echo "<p style=\"margin-bottom: 10px;\"><strong>Assignment deleted</strong></p>";
        break;

      default:
        $associates = $capacity_model->getDashboardData($_SESSION['user_id_department']);
    
        $averages = $capacity_model->getDeptWeeklyAverages($_SESSION['user_id_department']);
        $no_of_associates = count($users_model->listAssociates($_SESSION['user_id_department'], null));
        $assignments = $matters_model->getAssignments($_SESSION['user_id_department']); 
        $department = $departments_model->getOne($_SESSION['user_id_department']);
    
        $data = [
          'averages' => json_encode($averages),
          'associates' => $associates,
          'assignments' => $assignments,
          'department' => $department['department'],
          'partners' => $users_model->getPartners(),
          'matters' => $matters_model->getMyMatters($_SESSION['user_employeeNumber']),
        ];
        
        // print_r($data['partners']);
        // die();
        
        $headerData = $_SESSION;
        
        $headerData['department'] = $department['department'];
        $headerData['averages'] = $capacity_model->getHeaderAverages($_SESSION['user_id_department'])[0];
        $headerData['associates'] = count($associates);

        echo view('user/partner/header', $headerData);
        echo view('user/partner/associates/birds-eye', $data);
        echo view('user/partner/footer', $data);

        break;
    }
  }
  
  public function reports(){
    $users_model = new UsersModel();
    $capacity_model = new CapacityModel();
    $interests_model = new AssociateInterestsModel();
    $matters_model = new MattersModel();
    $matter_partners_model = new MatterPartnersModel();
    $assignments_model = new AssignmentsModel();
    $departments_model = new DepartmentsModel();
    
    $associates = $capacity_model->getDashboardData($_SESSION['user_id_department']);
    $averages = $capacity_model->getDeptWeeklyAverages($_SESSION['user_id_department']);
    $no_of_associates = count($users_model->listAssociates($_SESSION['user_id_department'], null));
    $assignments = $matters_model->getAssignments($_SESSION['user_employeeNumber']); 
    $department = $departments_model->getOne($_SESSION['user_id_department']);
    
    if(isset($_POST['reportDates'])){
      $reportDates = json_decode($_POST['reportDates']);
      $reportData = $matter_partners_model->getReport($reportDates->startDate, $reportDates->endDate);
    }else{
      $reportData = $matter_partners_model->getReport();
    }
    
    $ccoAssignments = [];
    $litigationAssignments = [];
    
    foreach($reportData as $assignment){
      if($assignment['added_by'] == 'E0038'){
        array_push($ccoAssignments, $assignment);
      }else{
        array_push($litigationAssignments, $assignment);
      }
    }
    
    $data = [
      'averages' => json_encode($averages),
      'associates' => $associates,
      'assignments' => $assignments,
      'department' => $department['department'],
      'partners' => $users_model->getPartners(),
      'matters' => $matters_model->getMyMatters($_SESSION['user_employeeNumber']),
    ];
    
    $headerData = $_SESSION;
    
    $headerData['department'] = $department['department'];
    $headerData['averages'] = $capacity_model->getHeaderAverages($_SESSION['user_id_department'])[0];
    $headerData['associates'] = count($associates);
    
    $styleArray = [
        'font' => [
            'bold' => true,
        ],
        'borders' => [
            'bottom' => [
                'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
            ],
        ]
    ];
    
    $spreadsheet = new Spreadsheet();
    $ccoWorkSheet = new \PhpOffice\PhpSpreadsheet\Worksheet\Worksheet($spreadsheet, 'CCO');
    $litigationWorkSheet = new \PhpOffice\PhpSpreadsheet\Worksheet\Worksheet($spreadsheet, 'Litigation');
    
    $spreadsheet->addSheet($ccoWorkSheet, 0);
    $spreadsheet->addSheet($litigationWorkSheet, 1);
    
    $sheet = $spreadsheet->getSheetByName('CCO');
    $sheet->setCellValue('A1', 'Matter');
    $sheet->setCellValue('B1', 'Requirement');
    $sheet->setCellValue('C1', 'Partner(s)');
    $sheet->setCellValue('D1', 'Associate(s)');
    $sheet->getStyle('A1:D1')->applyFromArray($styleArray);
    
    $sheet2 = $spreadsheet->getSheetByName('Litigation');
    $sheet2->setCellValue('A1', 'Matter');
    $sheet2->setCellValue('B1', 'Requirement');
    $sheet2->setCellValue('C1', 'Partner(s)');
    $sheet2->setCellValue('D1', 'Associate(s)');
    $sheet2->getStyle('A1:D1')->applyFromArray($styleArray);
    
    if(count($ccoAssignments) != 0){
      for($i=0; $i < count($ccoAssignments); $i++){
        $matterCellAddress = 'A' . ( $i + 2);
        $requirementCellAddress = 'B' . ( $i + 2);
        $partnersCellAddress = 'C' . ( $i + 2);
        $associatesCellAddress = 'D' . ( $i + 2);
        
        $sheet->setCellValue($matterCellAddress, $ccoAssignments[$i]['details']);
        $sheet->setCellValue($requirementCellAddress, $ccoAssignments[$i]['requirement']);
        $sheet->setCellValue($partnersCellAddress, $ccoAssignments[$i]['partners']);
        $sheet->setCellValue($associatesCellAddress, $ccoAssignments[$i]['associates']);
      }
    }
    
    if(count($litigationAssignments) != 0){
      for($i=0; $i < count($litigationAssignments); $i++){
        $matterCellAddress = 'A' . ( $i + 2);
        $requirementCellAddress = 'B' . ( $i + 2);
        $partnersCellAddress = 'C' . ( $i + 2);
        $associatesCellAddress = 'D' . ( $i + 2);
        
        $sheet2->setCellValue($matterCellAddress, $litigationAssignments[$i]['details']);
        $sheet2->setCellValue($requirementCellAddress, $litigationAssignments[$i]['requirement']);
        $sheet2->setCellValue($partnersCellAddress, $litigationAssignments[$i]['partners']);
        $sheet2->setCellValue($associatesCellAddress, $litigationAssignments[$i]['associates']);
      }
    }
    
    $writer = new Xlsx($spreadsheet);
    
    if(isset($_POST['reportDates'])){
      $reportFileName = 'reports/DatedReport-' . $reportDates->startDate . '-' . $reportDates->endDate . '-' . $_SESSION['user_employeeNumber'] . '.xlsx';
      $writer->save($reportFileName);
      echo '<p>
            <a href="' . base_url('/' . $reportFileName) . '" class="btn-link btn-link-grey">
              <span><i class="cil-cloud-download"></i></span>Download custom report' . '</a>
            </p>';
    }else{
      $writer->save('reports/Report.xlsx');
      
      echo view('user/partner/header', $headerData);
      echo view('user/partner/report');
      echo view('user/partner/footer', $data);
    }
  }
}
