<?php

namespace App\Controllers;

use App\Models\UsersModel;
use App\Models\AssociateInterestsModel;
use App\Models\PracticeAreasModel;
use App\Models\AreasOfInterestModel;
use App\Models\CapacityModel;

use CodeIgniter\Controller;

$session = session();

class User extends Controller
{
  // show profile
  public function showProfile()
  {
    $users_model = new UsersModel();
    $practice_areas_model = new PracticeAreasModel();
    $areas_of_interest_model = new AreasOfInterestModel();
    $interests_model = new AssociateInterestsModel();
    $capacity_model = new CapacityModel();
    

    // list ids to omit from areas of interest
    $omit = [2, 8, 10, 13, 14];

    // get today's capacity
    $capacity_today = $capacity_model->getTodaysCapacity($_SESSION['user_employeeNumber']);

    $data = [
      'departments' => $practice_areas_model->getPracticeAreas($omit),
      'capacity_today' => $capacity_today['capacity'],
      'interests' => $interests_model->getMyInterests($_SESSION['user_employeeNumber']),
      'partners' => $users_model->getPartners(),
      'areas' => $areas_of_interest_model->getAll(),
    ];

    // send data to client
    setcookie('capacityToday', $capacity_today['capacity']);

    echo view('templates/header', $data);
    echo view('user/associate/profile', $data);
    echo view('templates/footer', $data);
  }

  // add/edit interests
  public function myInterests($category)
  {
    $interests_model = new AssociateInterestsModel();

    if ($this->request->getPost('id')) {
      $data = ['id' => $this->request->getPost('id')];
    }

    switch ($category) {
      case 'areas': // areas
        $data['areas'] = $this->request->getPost('areas');
        break;

      case 'experiences': // experiences
        $data['experience'] = $this->request->getPost('experiences');
        break;

      default:
        // partners
        $data['partners'] = $this->request->getPost('partners');
        break;
    }

    $data['id_user'] = $this->request->getPost('id_user');

    $interests_model->save($data);

    $interests = $interests_model->getMyInterests($_SESSION['user_employeeNumber']);

    $display_data = [
      'partners' => json_decode($interests[0]['partners']),
      'areas' => json_decode($interests[0]['areas']),
      'experiences' => json_decode($interests[0]['experience']),
    ];

    $update_view = 'selected-' . $category;

    echo view('user/associate/' . $update_view, $display_data);
  }

  // view capacity history
  public function viewCapacityHistory()
  {
    echo view('templates/header');
    echo view('user/capacity/history');
    echo view('templates/footer');
  }

  // view capacity report
  public function viewCapacityReport()
  {
    echo view('templates/header');
    echo view('user/capacity/report');
    echo view('templates/footer');
  }

  // view new assignments
  public function viewNewAssignments()
  {
    echo view('templates/header');
    echo view('user/assignments/new');
    echo view('templates/footer');
  }

  // view current assignments
  public function viewCurrentAssignments()
  {
    echo view('templates/header');
    echo view('user/assignments/current');
    echo view('templates/footer');
  }

  // view asignment report
  public function viewAssignmentReport()
  {
    echo view('templates/header');
    echo view('user/assignments/report');
    echo view('templates/footer');
  }
}
