<?php

namespace App\Models;

use CodeIgniter\Model;

class CapacityModel extends Model
{
  protected $table = 'capacity';
  protected $primaryKey = 'id';
  protected $useAutoIncrement = true;
  protected $returnType = 'array';
  protected $useTimestamps = true;
  protected $createdField = 'created_at';
  protected $updatedField = 'updated_at';
  protected $allowedFields = ['id_user', 'capacity', 'capacity_date'];

  public function getAll()
  {
    return $this->findAll();
  }

  // get capacity for a single user
  public function getMyCapacity($employeeNumber)
  {
    return $this->query(
      "SELECT 
      id, capacity, DATE_FORMAT(capacity_date, '%a') as capacity_day, 
      @rownum := @rownum + 1 as capacity_index 
      FROM 
      capacity, (select @rownum := 0) r 
      WHERE 
      YEAR(capacity_date) = YEAR(CURDATE()) AND
      DATE_FORMAT(capacity_date, '%u') >= DATE_FORMAT(CURDATE(), '%u') AND
      DATE_FORMAT(capacity_date, '%a') != 'Sat' AND
      DATE_FORMAT(capacity_date, '%a') != 'Sun' AND
      id_user = '$employeeNumber'"
    )->getResultArray();
  }

  // get today's capacity for a single user
  public function getTodaysCapacity($id_user)
  {
    return $this->query(
      "SELECT capacity from capacity where capacity_date = CURDATE() and id_user = '$id_user'"
    )->getRowArray();
  }

  // show particular day's capacity for all
  public function getDaysCapacity($day)
  {
    $query = "SELECT
    u.employeeNumber, u.name, d.department, c.capacity
    from
    users as u, departments as d, capacity as c
    where
    u.id_department = d.id and
    u.employeeNumber = c.id_user and
    u.id_org_level in (6,8,10,11,12,13,14) AND
    u.status != 'Exited' AND
    DATE_FORMAT(capacity_date, '%u') >= DATE_FORMAT(CURDATE(), '%u') AND
    DATE_FORMAT(c.capacity_date, '%W') = '$day'";

    return $this->query($query)->getResultArray();
  }

  // get week's capacity
  public function getWeeksCapacity($week, $id_department)
  {
    $sql = "SELECT
    u.employeeNumber, u.name, d.department, o.level,
    MAX(CASE WHEN DATE_FORMAT(c.capacity_date, '%a') = 'Mon' then c.capacity END) as Mon,
    MAX(CASE WHEN DATE_FORMAT(c.capacity_date, '%a') = 'Tue' then c.capacity END) as Tue,
    MAX(CASE WHEN DATE_FORMAT(c.capacity_date, '%a') = 'Wed' then c.capacity END) as Wed,
    MAX(CASE WHEN DATE_FORMAT(c.capacity_date, '%a') = 'Thu' then c.capacity END) as Thu,
    MAX(CASE WHEN DATE_FORMAT(c.capacity_date, '%a') = 'Fri' then c.capacity END) as Fri
    FROM 
    users as u, capacity as c, departments as d, org_levels as o
    WHERE
    u.employeeNumber = c.id_user AND
    u.id_department = d.id AND
    u.status != 'Exited' AND
    u.id_org_level = o.id";

    if ($id_department) {
      $sql .= " AND u.id_department = $id_department";
    }

    if ($week) {
      $sql .= " AND DATE_FORMAT(c.capacity_date, '%u') >= $week";
    } else {
      $sql .=
        " AND DATE_FORMAT(c.capacity_date, '%u') >= DATE_FORMAT(CURDATE(), '%u')";
    }

    $sql .= ' GROUP BY u.name, u.employeeNumber, d.department, o.level ORDER BY u.name';

    return $this->query($sql)->getResultArray();
  }

  // get graph data
  public function getCapacityBillables($id_user)
  {
    $sql = "SELECT * FROM associate_graph_data WHERE id_user = '$id_user'";
    
    return $this->query($sql)->getResultArray();
  }

  // weekly billables and capacities totals (for graphing)
  public function getDeptWeeklyAverages($id_department)
  {
    $sql = "SELECT * FROM dept_weekly_averages";

    if ($id_department) {
      $sql .= " WHERE id = $id_department";
    }

    return $this->query($sql)->getResultArray();
  }
  
  // averages to display in header
  public function getHeaderAverages($id_department){
    $sql = "SELECT * from dept_three_week_avg";
    
    if ($id_department) {
      $sql .= "  WHERE id = $id_department";
    }
    
    return $this->query($sql)->getResultArray();
  }

  // dashboard data
  public function getDashboardData($id_department)
  {
    $sql = "SELECT * from dashboard_data WHERE id_department = $id_department ORDER BY name";
    return $this->query($sql)->getResultArray();
  }
  
  // get year's capacity for the API
  public function getYearsCapacity()
  {
    $sql = "SELECT id_user, week_no, weekly_capacity capacity FROM associate_graph_data ORDER BY id_user, week_no";
    return $this->query($sql)->getResultArray();
  } 
  
  // get year's capacity for the API
  public function getLawyerCapacity($employeeNumber)
  {
    $sql = "SELECT id_user, week_no, weekly_capacity capacity FROM associate_graph_data WHERE id_user = '$employeeNumber' ORDER BY week_no";
    
    return $this->query($sql)->getResultArray();
  } 
}
