<?php

namespace App\Models;

use CodeIgniter\Model;

class UsersModel extends Model
{
  protected $table = 'users';
  protected $primaryKey = 'id';
  protected $useAutoIncrement = true;
  protected $returnType = 'array';
  protected $allowedFields = ['status'];

  public function getAll()
  {
    return $this->findAll();
  }

  public function getOne($id)
  {
    return $this->find($id);
  }

  // list all users for admin
  public function listAssociates($id_department, $employeeNumber)
  {
    $sql = "SELECT u.employeeNumber, u.name, u.email, u.status, o.level, d.department
    FROM
    users as u,
    org_levels as o, departments as d
    WHERE
    u.id_org_level in (6, 8, 10, 11, 12, 13, 14) AND
    u.id_org_level = o.id AND
    u.status != 'Exited' AND
    u.id_department = d.id";

    if ($id_department) {
      $sql .= " AND u.id_department = $id_department";
    }

    if ($employeeNumber) {
      $sql .= " AND u.employeeNumber = '$employeeNumber'";
    }

    $sql .= ' ORDER BY u.name';

    return $this->query($sql)->getResultArray();
  }

  // get details for admin view profile page
  public function getProfileDetails($employeeNumber)
  {
    $sql = "SELECT u.name, u.email, u.status, o.level, d.department
            FROM
            users as u,
            org_levels as o, departments as d
            WHERE
            u.id_org_level = o.id AND
            u.id_department = d.id AND
            u.employeeNumber = '$employeeNumber'";

    return $this->query($sql)->getRowArray();
  }

  public function getByEmail($email)
  {
    return $this->query(
      "SELECT * from users where email = '$email'"
    )->getRowArray();
  }
  
  public function getByEmployeeNumber($employeeNumber)
  {
    return $this->query(
      "SELECT * from users where employeeNumber = '$employeeNumber'"
    )->getRowArray();
  }

  public function getPartners()
  {
    return $this->select('employeeNumber, name')
      ->whereIn('id_org_level', [1, 3, 4])
      ->orderBy('name')
      ->findAll();
  }

  // get areas and partners of interest
  public function getInterests($employeeNumber)
  {
    $sql = "SELECT ma.areas_of_interest, mp.partners
            FROM
            users as u, my_areas_of_interest as ma, my_partners as mp,
            org_levels as o, departments as d
            WHERE
            u.employeeNumber = ma.id_user AND
            u.employeeNumber = mp.id_user AND
            u.id_org_level = o.id AND
            u.id_department = d.id AND
            u.id = '$employeeNumber'";

    return $this->query($sql)->getRowArray();
  }

  // get experience
  public function getExperience($employeeNumber)
  {
    $sql = "SELECT d.department, me.description
            FROM
            users as u, my_experience as me, departments as d
            WHERE
            u.employeeNumber = me.id_user AND
            me.id_practice_area = d.id AND
            u.employeeNumber = '$employeeNumber'";

    return $this->query($sql)->getResultArray();
  }
}
