// define buttons
const btEditCapacity = document.querySelector('#edit-capacity')
const btFinishedEditingCapacity = document.querySelector(
  '#finished-editing-capacity'
)
const btEditNextWeek = document.querySelector('#edit-next-week')
const btEditNextWeekFull = document.querySelector(
  '#edit-capacity-next-week-full'
)
const btEditNextWeekSome = document.querySelector(
  '#edit-capacity-next-week-some'
)
const btEditNextWeekNone = document.querySelector(
  '#edit-capacity-next-week-none'
)

// define capacity fields
const capacityFields = document.querySelectorAll('.dashboard-field')

// define capacity stat containers
const capacityStatOne = document.querySelector('#capacity-stat-1')
const capacityStatTwo = document.querySelector('#capacity-stat-2')
const capacityStatThree = document.querySelector('#capacity-stat-3')
const capacityStatFour = document.querySelector('#capacity-stat-4')
const capacityStatFive = document.querySelector('#capacity-stat-5')

const nextWeekStatus = document.querySelector('#next-week-status')
const capacityNextWeek = document.querySelector('#capacity-stat-next-week')
const nextWeekEdits = document.querySelector('#next-week-edits')

// show edit capacity fields
const showEditFields = () => {
  // show the fields
  capacityFields.forEach(capacityField => {
    if (!capacityField.classList.contains('has-passed-field')) {
      capacityField.classList.remove('hide')
      capacityField.classList.add('animate__flipInX')
    }

    capacityField.addEventListener('animationend', () => {
      if (capacityField.classList.contains('animate__flipInX')) {
        capacityField.classList.remove('animate__flipInX')
      }
    })

    // hide the edit buttom and show the done button
    btEditCapacity.classList.add('animate__zoomOut')
    btFinishedEditingCapacity.classList.remove('hide')
    btFinishedEditingCapacity.classList.add('animate__zoomIn')
  })

  // place focus on first field
  document.querySelector('#day-1-value').focus()
}

// hide edit capacity fields
const hideEditFields = () => {
  capacityFields.forEach(capacityField => {
    capacityField.classList.add('animate__flipOutX')

    capacityField.addEventListener('animationend', () => {
      if (capacityField.classList.contains('animate__flipOutX')) {
        capacityField.classList.remove('animate__flipOutX')
        capacityField.classList.add('hide')
      }
    })

    // hide the done button and show the edit button
    btFinishedEditingCapacity.classList.add('animate__zoomOut')
    btEditCapacity.classList.remove('hide')
    btEditCapacity.classList.add('animate__zoomIn')
  })
}

//show edit fields
if (btEditCapacity) {
  btEditCapacity.addEventListener('click', () => showEditFields())

  // clean up the button animations
  btEditCapacity.addEventListener('animationend', () => {
    if (btEditCapacity.classList.contains('animate__zoomIn')) {
      btEditCapacity.classList.remove('animate__zoomIn')
    }

    if (btEditCapacity.classList.contains('animate__zoomOut')) {
      btEditCapacity.classList.remove('animate__zoomOut')
      btEditCapacity.classList.add('hide')
    }
  })
}

// if the user has entered a value greater than 8, default to 8
const validateCapacity = value => (value > 8 ? 8 : value)

// hide fields and submit data
if (btFinishedEditingCapacity) {
  btFinishedEditingCapacity.addEventListener('click', () => {
    // change display values
    capacityStatOne.innerText = validateCapacity(
      document.querySelector('#day-1-value').value
    )
    capacityStatTwo.innerText = validateCapacity(
      document.querySelector('#day-2-value').value
    )
    capacityStatThree.innerText = validateCapacity(
      document.querySelector('#day-3-value').value
    )
    capacityStatFour.innerText = validateCapacity(
      document.querySelector('#day-4-value').value
    )
    capacityStatFive.innerText = validateCapacity(
      document.querySelector('#day-5-value').value
    )

    hideEditFields()

    // data submission
    const capacityData = new FormData()

    const data = [
      {
        value: validateCapacity(document.querySelector('#day-1-value').value),
        id: document.querySelector('#capacity-id-1').innerText
      },
      {
        value: validateCapacity(document.querySelector('#day-2-value').value),
        id: document.querySelector('#capacity-id-2').innerText
      },
      {
        value: validateCapacity(document.querySelector('#day-3-value').value),
        id: document.querySelector('#capacity-id-3').innerText
      },
      {
        value: validateCapacity(document.querySelector('#day-4-value').value),
        id: document.querySelector('#capacity-id-4').innerText
      },
      {
        value: validateCapacity(document.querySelector('#day-5-value').value),
        id: document.querySelector('#capacity-id-5').innerText
      }
    ]

    capacityData.append('capacity', JSON.stringify(data))

    //show notification
    notify()

    $.ajax({
      url: '/submit-capacity',
      type: 'post',
      data: capacityData,
      processData: false,
      contentType: false,

      success: function (response) {
        $('#notification-message').html(response)

        hideNotification()
      }
    })
  })

  // clean up animations
  btFinishedEditingCapacity.addEventListener('animationend', () => {
    if (btFinishedEditingCapacity.classList.contains('animate__zoomIn')) {
      btFinishedEditingCapacity.classList.remove('animate__zoomIn')
    }

    if (btFinishedEditingCapacity.classList.contains('animate__zoomOut')) {
      btFinishedEditingCapacity.classList.remove('animate__zoomOut')
      btFinishedEditingCapacity.classList.add('hide')
    }
  })
}

// edit next week capacity
if (btEditNextWeek) {
  btEditNextWeek.addEventListener('click', () => {
    nextWeekStatus.classList.add('animate__fadeOut')

    nextWeekEdits.classList.remove('hide')
    nextWeekEdits.classList.add('animate__fadeIn')
  })

  cleanUpAnimation(nextWeekStatus, 'animate__fadeIn', 'animate__fadeOut')
  cleanUpAnimation(nextWeekEdits, 'animate__fadeIn', 'animate__fadeOut')
}

const submitNextWeek = capacity => {
  capacityNextWeek.textContent = capacity

  //show notification
  notify()

  const capacityData = new FormData()
  capacityData.append('capacity', capacity)
  capacityData.append('id', capacityNextWeek.dataset.capacityId)

  $.ajax({
    url: '/submit-next-week-capacity',
    type: 'post',
    data: capacityData,
    processData: false,
    contentType: false,

    success: function (response) {
      // hide edit buttons
      nextWeekEdits.classList.add('animate__fadeOut')
      nextWeekStatus.classList.remove('hide')
      nextWeekStatus.classList.add('animate__fadeIn')

      $('#notification-message').html(response)

      hideNotification()
    }
  })
}

// change status to full
if (btEditNextWeekFull) {
  btEditNextWeekFull.addEventListener('click', () =>
    submitNextWeek('Full capacity')
  )
}

// change status to some
if (btEditNextWeekSome) {
  btEditNextWeekSome.addEventListener('click', () =>
    submitNextWeek('Some capacity')
  )
}

// change status to none
if (btEditNextWeekNone) {
  btEditNextWeekNone.addEventListener('click', () =>
    submitNextWeek('No capacity')
  )
}

// toggle availability status

const dashboard = document.querySelector('#dashboard')
const dashboardAway = document.querySelector('#dashboard-away')
const btToggleStatus = document.querySelector('#toggle-status')
const btReactivateDashboard = document.querySelector('#reactivate-dashboard')
const userId = btToggleStatus.dataset.userId
const statusLabel = document.querySelector('#available-status').textContent

let available = statusLabel === 'Available'

const toggleStatus = userId => {
  notify()

  available = !available

  if (!available) {
    document.querySelector('#available-status').textContent = 'Away'
    document.querySelector('#available-status-icon').innerHTML =
      '<i class="cil-flight-takeoff"></i>'

    dashboardAway.classList.remove('hide')
    dashboard.classList.add('hide')
  } else {
    document.querySelector('#available-status').textContent = 'Available'
    document.querySelector('#available-status-icon').innerHTML =
      '<i class="cil-laptop"></i>'

    dashboardAway.classList.add('hide')
    dashboard.classList.remove('hide')
  }

  const statusData = new FormData()
  statusData.append('status', available ? 'Available' : 'Away')
  statusData.append('userId', userId)

  $.ajax({
    url: '/set-status',
    type: 'post',
    data: statusData,
    processData: false,
    contentType: false,

    success: function (response) {
      $('#notification-message').html(response)
      hideNotification()
    }
  })
}

btToggleStatus.addEventListener('click', () => toggleStatus(userId))
btReactivateDashboard.addEventListener('click', () => toggleStatus(userId))

// tooltip
tippy('#toggle-status', {
  content: 'Click to toggle'
})
