const DateTime = luxon.DateTime
const dt = DateTime.now()

// define buttons
const btAddMatter = document.querySelector('#btAddMatter')
const btFilter = document.querySelector('#btFilter')
const btCloseAddMatter = document.querySelector('#btCloseAddMatter')
const btCloseEditMatter = document.querySelector('#btCloseEditMatter')
const btCloseDeleteMatter = document.querySelector('#btCloseDeleteMatter')
const btCloseAddTask = document.querySelector('#btCloseAddTask')
const btSubmitMatter = document.querySelector('#btSubmitMatter')
const btEditMatter = document.querySelector('#btEditMatter')
const btAddTask = document.querySelector('#btAddTask')
const editButtons = document.querySelectorAll('.cil-pen')
const taskButtons = document.querySelectorAll('.cil-library-add')
const archiveButtons = document.querySelectorAll('.cil-storage')
const deleteButtons = Array.from(document.querySelectorAll('.cil-trash'))
const btCancelDelete = document.querySelector('#cancel-delete')
const btConfirmDelete = document.querySelector('#confirm-delete')

// define containers
const filter = document.querySelector('#filter') // filter
// const mainContent = document.querySelector('#main-content') // main component
const mattersContainer = document.querySelector('#matters-container') // matters container
const mattersList = document.querySelector('#matters-list') // inbox list
const matters = document.querySelector('#matters') // main matter window
const addMatter = document.querySelector('#add-matter') // add matter form container
const editMatter = document.querySelector('#edit-matter') // edit matter form container
const deleteMatter = document.querySelector('#delete-matter') // delete matter form container
const frmAddMatter = document.querySelector('#frmAddMatter') // add matter form
const frmEditMatter = document.querySelector('#frmEditMatter') // edit matter form
const frmDeleteMatter = document.querySelector('#frmDeleteMatter') // delete matter form
const addTask = document.querySelector('#add-task') // add task
const frmAddTask = document.querySelector('#frmAddTask') // add task form

// show/hide filter
if (btFilter) {
  btFilter.addEventListener('click', () => toggleFilter())
  let filterState = false

  const toggleFilter = () => {
    filterState = !filterState

    if (filterState) {
      filter.classList.remove('hide')
      filter.classList.add('animate__fadeInRight')
    } else {
      filter.classList.add('animate__fadeOutRight')
    }
  }
}

// create matters list element
const createMatterListElement = (id, matterDetails, matterDate) => {
  const container = document.createElement('div')
  container.classList.add('list-item')
  container.setAttribute('data-show-matter-id', id)

  const description = document.createElement('p')
  description.classList.add('list-description')
  description.textContent = matterDetails.substring(0, 60) + ' ...'

  const meta = document.createElement('p')
  meta.classList.add('list-meta')
  meta.textContent = matterDate

  container.appendChild(description)
  container.appendChild(meta)

  mattersList.prepend(container)
}

// create main data item
const createMatterItem = (
  id,
  matterRequirements,
  matterDetails,
  matterDate
) => {
  const container = document.createElement('div')
  container.classList.add(
    'data-item',
    'animate_animated',
    'animate__fadeInRight'
  )
  container.setAttribute('id', `matter-${id}`)

  // header
  const contentHeader = document.createElement('div')
  contentHeader.classList.add('content-header')

  const contentActions = document.createElement('p')
  contentActions.classList.add('content-actions')
  contentActions.innerHTML = `<i title="Edit matter" class="cil-pen" data-matter-id="${id}"></i>
                              <i title="Archive matter" class="cil-storage" data-matter-id="${id}"></i>
                              <i title="Delete matter" class="cil-trash" data-matter-id="${id}"></i>`

  contentHeader.appendChild(contentActions)
  // END header

  // meta
  const meta = document.createElement('div')
  meta.classList.add('content-meta')
  meta.innerHTML = `<p class="meta-date"><i class="cil-calendar"></i> <strong>Date created: </strong>${matterDate}</p>`
  // END meta

  // description
  const contentDescription = document.createElement('div')
  contentDescription.classList.add('content-description')
  contentDescription.innerHTML = `<p>${matterDetails}</p><hr><h3>Requirements</h3><p>${matterRequirements}</p>`
  // END description

  container.appendChild(contentHeader)
  container.appendChild(meta)
  container.appendChild(contentDescription)

  matters.prepend(container)
}

// show selected matter
const matterSelectors = Array.from(mattersList.children)
matterSelectors.forEach(selector => {
  const matterToShow = document.querySelector(
    `#matter-${selector.dataset.showMatterId}`
  )

  selector.addEventListener('click', () => {
    // hide visible matter
    hideMatters(selector.dataset.showMatterId)

    // mark matter as clicked
    selector.classList.add('list-item-clicked')

    matterToShow.classList.remove('hide')
    matterToShow.classList.add('animate__fadeInRight')
  })
})

// hide data items
const hideMatters = visibleId => {
  const matters = Array.from(document.querySelectorAll('.data-item'))

  // unmark matter selector
  matterSelectors.forEach(selector => {
    if (selector.getAttribute('data-show-matter-id') !== visibleId) {
      selector.classList.remove('list-item-clicked')
    }
  })

  // hide previously selected matter
  matters.forEach(matter => {
    if (
      matter.classList.contains('animate__fadeInRight') &&
      matter.id !== `matter-${visibleId}`
    ) {
      matter.classList.remove('animate__fadeInRight')
      matter.classList.add('animate__zoomOut')
    }

    matter.addEventListener('animationend', () => {
      if (matter.classList.contains('animate__zoomOut')) {
        matter.classList.remove('animate__zoomOut')
        matter.classList.add('hide')
      }
    })
  })
}

// create task element
const createTaskElement = (matterId, title, duration) => {
  const tasks = document.querySelector(`#tasks-${matterId}`)

  const taskContainer = document.createElement('div')
  taskContainer.classList.add('task', 'unassigned')

  const taskDetails = document.createElement('div')
  taskDetails.classList.add('tg-container')

  const taskTitleContainer = document.createElement('div')
  taskTitleContainer.classList.add('tg-column', 'tg-col-12', 'no-bottom-margin')
  taskTitleContainer.innerHTML = `<p><strong>${title}</strong></p><hr>`

  const taskDurationContainer = document.createElement('div')
  taskDurationContainer.classList.add(
    'tg-column',
    'tg-col-3',
    'no-bottom-margin'
  )
  taskDurationContainer.innerHTML = `<p><i class="cil-clock"></i> ${duration} hours</p>`

  const taskStatusContainer = document.createElement('div')
  taskStatusContainer.classList.add('tg-column', 'tg-col-3', 'no-bottom-margin')
  taskStatusContainer.innerHTML = `<p class="badge new">New</p>`

  taskDetails.append(taskTitleContainer)
  taskDetails.append(taskDurationContainer)
  taskDetails.append(taskStatusContainer)
  taskContainer.append(taskDetails)

  tasks.append(taskContainer)
}

//show add matter form
btAddMatter.addEventListener('click', () => showForm(addMatter))

// hide add matter form
btCloseAddMatter.addEventListener('click', () => hideForm(addMatter))

// add new matter
btSubmitMatter.addEventListener('click', () => {
  const matterRequirements = frmAddMatter.querySelector(
    '#matter-requirements'
  ).value
  const matterDetails = frmAddMatter.querySelector('#matter-details').value

  const matterData = new FormData()
  matterData.append('requirement', matterRequirements)
  matterData.append('details', matterDetails)

  //show notification
  notify()

  // send data to db
  $.ajax({
    url: '/partner/matters/submit',
    type: 'post',
    data: matterData,
    processData: false,
    contentType: false,

    success: function (response) {
      $('#notification-message').html(response)

      const idMatter = $.cookie('idMatter')

      // add to matters list
      createMatterListElement(
        idMatter,
        matterDetails,
        dt.toLocaleString(DateTime.DATE_MED)
      )

      // add to main matters window
      createMatterItem(
        idMatter,
        matterRequirements,
        matterDetails,
        dt.toLocaleString(DateTime.DATE_MED)
      )

      hideMatters(idMatter)
      hideNotification()
    }
  })
  // end send data to db

  // reset form
  frmAddMatter.reset()

  // hide form and show data
  addMatter.classList.add('animate__zoomOut')
  mattersContainer.classList.remove('blurry')
})

// delete matters

// 1. show delete form
deleteButtons.forEach(btnDelete => {
  btnDelete.addEventListener('click', () => {
    const matterId = btnDelete.getAttribute('data-matter-id')

    // inject matter id into delete form
    frmDeleteMatter.querySelector('#matter-id').value = matterId

    // show delete confirmation form
    showForm(deleteMatter)

    // display matter title in delete form
    const matterTitle =
      btnDelete.parentElement.previousElementSibling.textContent
    frmDeleteMatter.querySelector('#delete-matter-title').textContent =
      matterTitle
  })
})

// 2. delete the matter
btConfirmDelete.addEventListener('click', () => {
  notify()

  const matterId = frmDeleteMatter.querySelector('#matter-id').value

  // delete matter from db
  $.ajax({
    url: `/partner/matters/delete/${matterId}`,
    type: 'get',
    // data: matterData,
    processData: false,
    contentType: false,

    success: function (response) {
      // show matter next to it
      const nextMatter = document.querySelector(
        `#matter-${matterId}`
      ).nextElementSibling

      nextMatter.classList.remove('hide')
      nextMatter.classList.add('animate__fadeInRight')

      // highlight selector for next matter
      const nextSelector = document.querySelector(
        `#matter-selector-${matterId}`
      ).nextElementSibling
      nextSelector.classList.add('list-item-clicked')

      // remove matter from main window
      document.querySelector(`#matter-${matterId}`).remove()

      // remove matter selector from list
      document.querySelector(`#matter-selector-${matterId}`).remove()

      $('#notification-message').html(response)

      hideForm(deleteMatter)
      hideNotification()
    }
  })
  // end delete matter from db
})

// hide delete matter form
btCloseDeleteMatter.addEventListener('click', () => hideForm(deleteMatter))
btCancelDelete.addEventListener('click', () => hideForm(deleteMatter))

// edit matters
// 1. show edit form
editButtons.forEach(btnEdit => {
  btnEdit.addEventListener('click', () => {
    const matterId = btnEdit.getAttribute('data-matter-id')

    // get current values
    const matterRequirements = document.querySelector(
      `#matter-name-${matterId}`
    ).textContent
    const matterClient = document.querySelector(
      `#matter-client-${matterId}`
    ).textContent
    const matterDetails = document.querySelector(
      `#matter-description-${matterId}`
    ).textContent

    // inject matter details into edit form
    frmEditMatter.querySelector('#edit-matter-name').value = matterRequirements
    frmEditMatter.querySelector('#edit-matter-client').value = matterClient
    frmEditMatter.querySelector('#edit-matter-description').value =
      matterDetails

    // inject matter id into delete form
    frmEditMatter.querySelector('#edit-matter-id').value = matterId

    // show edit form
    showForm(editMatter)
  })
})

// 2. submit changes
btEditMatter.addEventListener('click', () => {
  notify()

  // get form values
  const matterRequirements =
    frmEditMatter.querySelector('#edit-matter-name').value
  const matterClient = frmEditMatter.querySelector('#edit-matter-client').value
  const matterDetails = frmEditMatter.querySelector(
    '#edit-matter-description'
  ).value
  const matterId = frmEditMatter.querySelector('#edit-matter-id').value

  // send matter values to db
  const matterData = new FormData()
  matterData.append('id', matterId)
  matterData.append('name', matterRequirements)
  matterData.append('client', matterClient)
  matterData.append('description', matterDetails)

  $.ajax({
    url: `/partner/matters/edit/${matterId}`,
    type: 'post',
    data: matterData,
    processData: false,
    contentType: false,

    success: function (response) {
      // update matter view
      document.querySelector(`#matter-name-${matterId}`).textContent =
        matterRequirements

      document.querySelector(`#matter-client-${matterId}`).textContent =
        matterClient

      document.querySelector(`#matter-description-${matterId}`).textContent =
        matterDetails

      // update matters list
      document.querySelector(`#matter-selector-name-${matterId}`).textContent =
        matterRequirements

      document.querySelector(
        `#matter-selector-description-${matterId}`
      ).textContent = matterDetails.substring(0, 60) + ' ...'

      $('#notification-message').html(response)

      // reset form
      frmEditMatter.reset()

      hideForm(editMatter)
      hideNotification()
    }
  })
  // end send data to db
})

// hide edit matter form
btCloseEditMatter.addEventListener('click', () => hideForm(editMatter))

// add task

// 1. show add task form
taskButtons.forEach(btnTask => {
  btnTask.addEventListener('click', () => {
    const matterId = btnTask.getAttribute('data-matter-id')

    // inject matter id into delete form
    frmAddTask.querySelector('#task-matter-id').value = matterId

    // show delete confirmation form
    showForm(addTask)
  })
})

// add task to matter view and to the db
btAddTask.addEventListener('click', () => {
  notify()

  const taskName = frmAddTask.querySelector('#task-name').value
  const taskDuration = frmAddTask.querySelector('#task-duration').value
  const taskDescription = frmAddTask.querySelector('#task-description').value
  const taskAssignTo = frmAddTask.querySelector('#task-assign-to').value
  const matterId = frmAddTask.querySelector('#task-matter-id').value

  const taskData = new FormData()
  taskData.append('task', taskName)
  taskData.append('description', taskDescription)
  taskData.append('duration', taskDuration)
  taskData.append('id_associate', taskAssignTo)
  taskData.append('id_matter', matterId)

  $.ajax({
    url: `/partner/matters/addtask/${matterId}`,
    type: 'post',
    data: taskData,
    processData: false,
    contentType: false,

    success: function (response) {
      // add task to matter view
      createTaskElement(matterId, taskName, taskDuration)

      $('#notification-message').html(response)

      // reset form
      frmAddTask.reset()

      hideForm(addTask)
      hideNotification()
    }
  })
})

// hide add task form
btCloseAddTask.addEventListener('click', () => hideForm(addTask))

// cleanup
cleanUpAnimation(addMatter, 'animate__zoomIn', 'animate__zoomOut')
cleanUpAnimation(editMatter, 'animate__zoomIn', 'animate__zoomOut')
cleanUpAnimation(deleteMatter, 'animate__zoomIn', 'animate__zoomOut')
cleanUpAnimation(addTask, 'animate__zoomIn', 'animate__zoomOut')
cleanUpAnimation(filter, 'animate__fadeInRight', 'animate__fadeOutRight')
