// PARTNERS JS FILE

// define buttons
const btFinishedEditing = document.querySelector('#finished-editing-partners')
const btEditPartners = document.querySelector('#edit-partners')
const btCloseEditPartners = document.querySelector('#close-edit-partners')

// define containers
const partnersListContainer = document.querySelector('#partners-list-container') // animated container form all partners

// show the list of partners on clicking the edit button
if (btEditPartners) {
  // conditional prevents render blocking errors on pages that don't have these elements
  btEditPartners.addEventListener('click', e => {
    e.preventDefault()

    showForm(partnersListContainer)
  })
}

// close partners list
if (btCloseEditPartners) {
  // conditional prevents render blocking errors on pages that don't have these elements
  btCloseEditPartners.addEventListener('click', () =>
    hideForm(partnersListContainer)
  )
}

// 1. get list of selected partners
const selectedPartnerContainers = Array.from(
  document.querySelectorAll('.selected-partner p')
)

let selectedPartners = selectedPartnerContainers.map(container => ({
  id: container.dataset.partnerId,
  name: container.innerText
}))

// 2. get list of all partners and check to see if any of them are selected
const allPartners = Array.from(document.querySelectorAll('.partner-checkbox'))

allPartners.forEach(partner => {
  const partnerId = partner.dataset.id
  const partnerStatus = partner.nextElementSibling.nextElementSibling
  const partnerLabel = partner.parentElement

  if (
    selectedPartners.filter(selectedPartner => selectedPartner.id === partnerId)
      .length !== 0
  ) {
    partnerLabel.classList.add('choice-selected')
    partnerStatus.innerHTML = "<i class='cil-check-alt'></i>"
    partner.checked = true
  }

  // add/remove from selection
  partner.addEventListener('click', () => {
    if (partner.checked === true) {
      // add partner to selectedPartners
      selectedPartners.push({
        id: partnerId,
        name: partner['value']
      })

      // change icon of status span
      partnerStatus.innerHTML = "<i class='cil-check-alt'></i>"
      partnerLabel.classList.add('choice-selected')
    } else {
      // partner has already been selected, clicking again removes them from the list
      selectedPartners = selectedPartners.filter(
        partnerToCheck => partnerToCheck.id !== partnerId
      )

      // change icon of status span
      partnerStatus.innerHTML = "<i class='cil-x-circle'></i>"
      partnerLabel.classList.remove('choice-selected')
    }
  })
})

// submit selected partners
if (btFinishedEditing) {
  btFinishedEditing.addEventListener('click', () => {
    //show notification
    notify()

    // save selected parteners to database
    const partnerData = new FormData()
    partnerData.append('partners', JSON.stringify(selectedPartners))
    partnerData.append('id_user', clData.user.id)

    // if the selected parters is an edit, an id exits ... include it in the post data to facilitate an update instead of an insertion
    if (document.querySelector('#interests-id').innerText) {
      partnerData.append(
        'id',
        document.querySelector('#interests-id').innerText
      )
    }

    // hide selection form
    hideForm(partnersListContainer)

    $.ajax({
      url: '/submit-interests/partners',
      type: 'post',
      data: partnerData,
      processData: false,
      contentType: false,

      success: function (response) {
        $('#selected-partners-container').html(response)
        $('#notification-message').html(
          `<p><i class="cil-task"></i></p><p>Partners of interest updated</p>`
        )
        hideNotification()
      }
    })
  })
}
