// PRACTICE AREAS JS FILE

// define buttons
const btFinishedPracticeAreas = document.querySelector('#finished-editing-pas')
const btEditPracticeAreas = document.querySelector('#edit-pas')
const btCloseEditPas = document.querySelector('#close-edit-areas-of-interest')

// define containers
const emptyPracticeAreas = document.querySelector('#empty-pas') // when no pa has been selected
const pasListContainer = document.querySelector('#pas-list-container') // animated container for all pas
const pasList = document.querySelector('#pas-list') // flex container for all pas
const selectedPasContainer = document.querySelector('#selected-pas-container') // animated container for selected pas
const selectedPas = document.querySelector('#selected-pas') // flex container for selected pas

//load practice areas from the db
const practiceAreas = JSON.parse($.cookie('practiceAreas'))

// load SELECTED practice areas from the db
const myPracticeAreas = JSON.parse($.cookie('myPracticeAreas'))

// save SELECTED practice areas from the db into the local storage object
if (myPracticeAreas) {
  clData = {
    ...clData,
    myPracticeAreas: JSON.parse(myPracticeAreas.practice_areas)
  }
}

// save to local storage
localStorage.setItem('clData', JSON.stringify(clData))

// =========================================================================

// initialise empty selected pas array
let areasOfInterest

// if main data object has a list of myPractice areas
if (clData.myPracticeAreas !== undefined) {
  // 1. add them to areasOfInterest array
  areasOfInterest = clData.myPracticeAreas

  // 2. display them to the user
  clData.myPracticeAreas.forEach(pa => {
    const paContainer = document.createElement('div')
    paContainer.setAttribute('id', `selected-pa-${pa.id}`)
    paContainer.classList.add('tg-column', 'tg-col-6', 'no-bottom-margin')

    const paTag = document.createElement('div')
    paTag.classList.add('tag')
    paTag.innerHTML = `<p>${pa.department}</p>`

    paContainer.appendChild(paTag)

    selectedPas.appendChild(paContainer)

    // 3. hide the empty div
    emptyPracticeAreas.classList.add('hide')
  })
} else {
  // user hasn't selected any pas
  areasOfInterest = []

  // show the list of pas
  pasList.classList.remove('hide')
}

// SECTION ONE
// create list of practice areas for user to select from
practiceAreas.forEach(practiceArea => {
  const { id, department } = practiceArea

  // create child elements
  let paNode = document.createElement('div')
  paNode.classList.add('tg-column', 'tg-col-6', 'no-bottom-margin')
  paNode.setAttribute('id', `practice-area-${id}`)

  // create checkbox
  let paCheckbox = document.createElement('input')
  paCheckbox.setAttribute('type', 'checkbox')
  paCheckbox.setAttribute('id', `pa-check-${id}`)
  paCheckbox.setAttribute('value', department)
  paCheckbox.setAttribute('data-id', id)
  paCheckbox.classList.add('pa-checkbox')

  //create span that will have the label text
  let paSpan = document.createElement('span')
  paSpan.appendChild(document.createTextNode(department))

  // create span that will show status
  let paStatusSpan = document.createElement('span')
  paStatusSpan.setAttribute('id', `pa-status-${id}`)
  paStatusSpan.innerHTML = "<i class='cil-x-circle'></i>"

  // create label
  let paLabel = document.createElement('label')
  paLabel.setAttribute('for', `pa-check-${id}`)
  paLabel.classList.add('choice-label')

  // add checkbox and practice area name to label
  paLabel.appendChild(paCheckbox)
  paLabel.appendChild(paSpan)
  paLabel.appendChild(paStatusSpan)

  // add label and checkbox to practice area node
  paNode.appendChild(paLabel)

  //check to see if pa has already been selected by the user
  if (areasOfInterest.filter(pa => pa.id === id).length !== 0) {
    paLabel.classList.add('choice-selected')
    paStatusSpan.innerHTML = "<i class='cil-check-alt'></i>"
    paCheckbox.checked = true
  }
})
// END SECTION ONE

// show the list of pas on clicking the edit button
btEditPracticeAreas.addEventListener('click', e => {
  e.preventDefault()

  pasListContainer.classList.remove('hide')
  pasListContainer.classList.add('animate__zoomIn')
})

// hide list of pas
btCloseEditPas.addEventListener('click', () => {
  pasListContainer.classList.add('animate__zoomOut')
})

// SECTION TWO
// make selection of preferred practice areas
const paControls = document.querySelectorAll('.pa-checkbox')

paControls.forEach(paControl => {
  // get the status span of the control
  const paID = paControl.getAttribute('data-id')
  const paStatus = document.querySelector(`#pa-status-${paID}`)

  paControl.addEventListener('click', () => {
    //create practice area tag
    const paContainer = document.createElement('div')
    paContainer.setAttribute('id', `selected-pa-${paControl['id']}`)
    paContainer.classList.add(
      'tg-column',
      'tg-col-6',
      'no-bottom-margin',
      'animate__animated',
      'animate__fadeIn'
    )

    const paTag = document.createElement('div')
    paTag.classList.add('tag')
    paTag.innerHTML = `<p>${paControl['value']}</p>`

    paContainer.appendChild(paTag)

    if (paControl['checked'] === true) {
      // add partner to partnersOfInterest
      areasOfInterest.push({
        id: paID,
        department: paControl['value']
      })

      // change icon of status span
      paStatus.innerHTML = "<i class='cil-check-alt'></i>"
      paStatus.parentNode.classList.add('choice-selected')

      // add practice area tag to selected partners list
      selectedPas.appendChild(paContainer)
    } else {
      // practice area has already been selected, clicking again removes them from the list
      areasOfInterest = areasOfInterest.filter(
        areaToCheck => areaToCheck.id !== paID
      )

      // change icon of status span
      paStatus.innerHTML = "<i class='cil-x-circle'></i>"
      paStatus.parentNode.classList.remove('choice-selected')

      // remove partner tag to selected partners list
      document.querySelector(`#selected-pa-${paID}`).remove()
    }

    // if the selection goes down to 0, show the empty div
    if (areasOfInterest.length === 0) {
      emptyPracticeAreas.classList.remove('hide')
    } else {
      emptyPracticeAreas.classList.add('hide')
    }
  })
})
// END SECTION TWO

// SECTION THREE
// add selected practice areas to the db & local storage
btFinishedPracticeAreas.addEventListener('click', () => {
  //show notification
  notify()

  // hide all pas list
  pasListContainer.classList.add('animate__zoomOut')

  // add selected pas to main data object
  clData = {
    ...clData,
    myPracticeAreas: areasOfInterest
  }

  // save to local storage
  localStorage.setItem('clData', JSON.stringify(clData))

  // save selected pas to database
  const paData = new FormData()
  paData.append('practice_areas', JSON.stringify(clData.myPracticeAreas))
  paData.append('id_user', clData.user.id)

  // if the selected pas is an edit, an id exits ... include it in the post data to facilitate an update instead of an insertion
  if (myPracticeAreas) {
    paData.append('id', myPracticeAreas.id)
  }

  $.ajax({
    url: '/submit-practice-areas',
    type: 'post',
    data: paData,
    processData: false,
    contentType: false,

    success: function (response) {
      $('#notification-message').html(response)
      hideNotification()
    }
  })
})
// TODO remove the below and use the global function
// clear animations
pasListContainer.addEventListener('animationend', () => {
  if (pasListContainer.classList.contains('animate__zoomIn')) {
    pasListContainer.classList.remove('animate__zoomIn')
  }

  if (pasListContainer.classList.contains('animate__zoomOut')) {
    pasListContainer.classList.remove('animate__zoomOut')
    pasListContainer.classList.add('hide')
  }
})
