// constants
const windowHeight = $(window).height()
const windowWidth = $(window).width()
const documentHeight = $(document).height()

// main js file
localStorage.clear()

// get data from localStorage
let clData = JSON.parse(localStorage.getItem('clData'))

// get user data
if ($.cookie('userData') !== undefined) {
  const userData = JSON.parse($.cookie('userData'))

  // add user data to main object, if it doesn't exist
  if (clData === null) {
    // add values to clData and save to local storage
    clData = {
      user: {
        id: userData.user_id,
        name: userData.user_name,
        email: userData.user_email
      }
    }

    localStorage.setItem('clData', JSON.stringify(clData))
  }
}

// show notification
const notify = () => {
  const notification = document.querySelector('#notification')

  // show notification
  notification.classList.remove('hide')
  notification.classList.add('animate__fadeInRight')

  cleanUpAnimation(notification, 'animate__fadeInRight', null)
}

// hide notification
const hideNotification = () => {
  setTimeout(() => {
    notification.classList.add('animate__fadeOutRight')

    cleanUpAnimation(notification, null, 'animate__fadeOutRight')
  }, 3000)
}

// clean up animations
const cleanUpAnimation = (element, entranceAnimation, exitAnimation) => {
  element.addEventListener('animationend', () => {
    if (element.classList.contains(entranceAnimation)) {
      element.classList.remove(entranceAnimation)
    }

    if (element.classList.contains(exitAnimation)) {
      element.classList.remove(exitAnimation)
      element.classList.add('hide')
    }
  })
}

$(document).ready(function () {
  // data tables
  $('#days-capacity, #weeks-capacity').DataTable()
  $('#table-assignments').DataTable({
    order: [[3, 'desc']],
    columns: [null, null, null, { width: '10%' }]
  })
})

// show mask
const mask = document.querySelector('#mask')

const showMask = () => {
  mask.style.width = `${windowWidth}px`
  mask.style.height = `${documentHeight}px`
  mask.classList.remove('hide')
  mask.classList.add('animate__zoomIn')
}

// hide mask
const hideMask = () => mask.classList.add('animate__zoomOut')

// show form
const showForm = form => {
  const formHeight = $(form).height()
  const formWidth = $(form).width()

  const positionTop = windowHeight / 2 - formHeight / 2
  const positionLeft = windowWidth / 2 - formWidth / 2

  form.style.top = `${positionTop}px`
  form.style.left = `${positionLeft}px`

  form.classList.remove('hide')
  form.classList.add('animate__zoomIn')

  cleanUpAnimation(form, 'animate__zoomIn', null)
}

// hide form
const hideForm = form => {
  form.classList.add('animate__zoomOut')
  cleanUpAnimation(form, null, 'animate__zoomOut')
}

// clean up animations
// cleanUpAnimation(mask, 'animate__zoomIn', 'animate__zoomOut')
